#include <PalmOS.h>

//assume max resourse size 64K so index will be uint16


#define IndexType		UInt16
#define MAX_INDICES		2048
#define IndexResType	'qidx'
#define QuoteResType	'tSTR'
#define SEPARATOR		'~'



#define dbg_(x) {char xs[9];StrIToH(xs,x);FrmCustomAlert(10024,xs,NULL,NULL);}



Boolean indexRes(DmOpenRef db,UInt16 id){
	MemHandle sh;
	char* s;
	MemHandle ih;
	IndexType* i;
	UInt16 j,c=0;
	UInt16 sz;
	
	//first check if index already exists and bail out if it does
	
	
	if((ih=DmGet1Resource(IndexResType,id))!=NULL){
		DmReleaseResource(ih);
		return false;
	}
	
	
	//now do some indexing
	
	i=MemPtrNew(sizeof(IndexType)*MAX_INDICES);
	
	if(!i)
		SysFatalAlert("Indexing failed: can't allocate index table");
	
	sh=DmGet1Resource(QuoteResType,id);
	
	
	if(!sh)
		SysFatalAlert("Indexing failed: can't get resource");
	
	sz=MemHandleSize(sh);		//could have used StrLen but why make it do all the work going through 64k of text?
	
	s=MemHandleLock(sh);
	
	
	
	for(j=0;j<sz;j++)
		if(s[j]==SEPARATOR)
			i[c++]=j+1;		//+1 so we point to actual next and not the newline characted
	
	
	MemHandleUnlock(sh);
	
	DmReleaseResource(sh);
	
	
	//c now contains the number of items in the index
	
	ih=DmNewResource(db,IndexResType,id,sz=c*sizeof(IndexType));
	
	if(!ih)
		SysFatalAlert("Indexing failed: can't allocate index resource");
	
	
	DmWrite(MemHandleLock(ih),0,i,sz);
	
	
	MemPtrFree(i);
	
	MemHandleUnlock(ih);
	
	DmReleaseResource(ih);
	
	
	return true;
}

DmOpenRef openDB(){
	LocalID LID;
	DmOpenRef db;
	UInt16 cn;
	
	
	SysCurAppDatabase(&cn,&LID);
	
	
	if(!LID)
		SysFatalAlert("LID unknown");
	
	
	db=DmOpenDatabase(0,LID,dmModeReadWrite);
	
	
	if(!db)
		SysFatalAlert("DB not opened");
		
	
	return db;
}

UInt16 getNumResources(){
	
	UInt16 j=1000;
	MemHandle h;
	
	while((h=DmGet1Resource(QuoteResType,j++))!=NULL)
		DmReleaseResource(h);
	
	return j-1001;
	
}

UInt16 getNumQuotes(UInt16 id){		//assume index exists
	MemHandle mh;
	UInt16 res;
	
	
	mh=DmGet1Resource(IndexResType,id);
	res=MemHandleSize(mh)/sizeof(IndexType);
	DmReleaseResource(mh);
	
	return res;
}

Boolean indexData(DmOpenRef db){
	UInt16 i,num;
	Boolean indexed=false;
	
	
	num=getNumResources();
	
	
	
	for(i=0;i<num;i++)
		indexed|=indexRes(db,i+1000);
	
	
	
	return indexed;
}

void compressData(){

}

void doOneTimeStuff(){
	DmOpenRef db;
	
	
	db=openDB();
	
	
	if(indexData(db))
		compressData(db);
		
	
	DmCloseDatabase(db);
	
}

UInt32 getHour(){
	return (TimGetSeconds())/(3600);
}

UInt32 getQuoteLen(char* c){	//length till next newline OR '\0'
	
	UInt32 i=0;
	
	
	while(c[i]!=SEPARATOR && c[i]!='\0')
		i++;
	
	
	return i;
}

void showQuoteByIndex(UInt16 r,UInt16 i,char* numStr){			//assume index exists
	
	MemHandle h;
	IndexType* idx;
	IndexType off;
	char* c;
	char* q;
	UInt32 L;

	
	h=DmGet1Resource(IndexResType,r);
	
	idx=MemHandleLock(h);
	
	off=idx[i];
	
	MemHandleUnlock(h);
	
	DmReleaseResource(h);
	
	h=DmGet1Resource(QuoteResType,r);
	
	c=MemHandleLock(h);
	
	
	c=(char*)(((UInt32)(c))+((UInt32)(off)));
	
	q=MemPtrNew((L=getQuoteLen(c))+1);
	
	StrNCopy(q,c,L);
	q[L]=0;
	
	MemHandleUnlock(h);
	
	DmReleaseResource(h);
	
	StrCat(numStr,")");
	FrmCustomAlert(10024,q,"\n(",numStr);
	
	MemPtrFree(q);
}

void showQuote(){
	
	UInt16 i,r;
	UInt32 d;
	UInt32 tn=0;
	char numStr[12];
	
	
	
	for(i=0;i<getNumResources();i++)
		tn+=getNumQuotes(i+1000);

	
	
	d=getHour()%tn;
	
	//d is now the serial quote number but we dont know what file it's in so we go through them
	//all and find one where it is still there
	
	StrIToA(numStr,d);
	
	r=1000;

	
	while((tn=getNumQuotes(r))<=d){
		r++;
		d-=tn;
	}
	
	
	//now r is the resource in which our quote is and d is the index into that resource	
	
	
	showQuoteByIndex(r,d,numStr);
	
		
}





UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	
	if(cmd==0){
		
		
		doOneTimeStuff();
		
		
		showQuote();
	}
	
	return 0;
}
